/**
 * @file
 * VuoList_VuoCurveEasing C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoCurveEasing_H
#define VuoList_VuoCurveEasing_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoCurveEasing VuoList_VuoCurveEasing
 * A list of @ref VuoCurveEasing elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoCurveEasing_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoCurveEasing elements.
 */
typedef const struct VuoList_VuoCurveEasing_struct { void *l; } * VuoList_VuoCurveEasing;
/// @}
#endif

/**
 * Creates a new list of @ref VuoCurveEasing elements.
 */
VuoList_VuoCurveEasing VuoListCreate_VuoCurveEasing(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoCurveEasing VuoListCopy_VuoCurveEasing(const VuoList_VuoCurveEasing list);

/**
 * Returns the @ref VuoCurveEasing at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoCurveEasing VuoListGetValue_VuoCurveEasing(const VuoList_VuoCurveEasing list, const unsigned long index);

/**
 * Changes the @ref VuoCurveEasing at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoCurveEasing(const VuoList_VuoCurveEasing list, const VuoCurveEasing value, const unsigned long index);

/**
 * Inserts the `VuoCurveEasing` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoCurveEasing(const VuoList_VuoCurveEasing list, const VuoCurveEasing value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoCurveEasing(VuoList_VuoCurveEasing list, const VuoCurveEasing value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoCurveEasing(VuoList_VuoCurveEasing list, const VuoCurveEasing value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoCurveEasing(VuoList_VuoCurveEasing list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoCurveEasing(VuoList_VuoCurveEasing list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoCurveEasing(VuoList_VuoCurveEasing list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoCurveEasing(VuoList_VuoCurveEasing list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoCurveEasing(VuoList_VuoCurveEasing list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoCurveEasing(VuoList_VuoCurveEasing list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoCurveEasing(VuoList_VuoCurveEasing list);

/**
 * Removes the `VuoCurveEasing` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoCurveEasing(VuoList_VuoCurveEasing list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoCurveEasing(const VuoList_VuoCurveEasing list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoCurveEasing VuoList_VuoCurveEasing_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoCurveEasing_getJson(const VuoList_VuoCurveEasing value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoCurveEasing_getSummary(const VuoList_VuoCurveEasing value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoCurveEasing VuoList_VuoCurveEasing_makeFromString(const char *str);
char * VuoList_VuoCurveEasing_getString(const VuoList_VuoCurveEasing value);
/// @}

/**
 * @}
 */

#endif
