/**
 * @file
 * VuoList_VuoLeapPointableType C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoLeapPointableType_H
#define VuoList_VuoLeapPointableType_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoLeapPointableType VuoList_VuoLeapPointableType
 * A list of @ref VuoLeapPointableType elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoLeapPointableType_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoLeapPointableType elements.
 */
typedef const struct VuoList_VuoLeapPointableType_struct { void *l; } * VuoList_VuoLeapPointableType;
/// @}
#endif

/**
 * Creates a new list of @ref VuoLeapPointableType elements.
 */
VuoList_VuoLeapPointableType VuoListCreate_VuoLeapPointableType(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoLeapPointableType VuoListCopy_VuoLeapPointableType(const VuoList_VuoLeapPointableType list);

/**
 * Returns the @ref VuoLeapPointableType at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoLeapPointableType VuoListGetValue_VuoLeapPointableType(const VuoList_VuoLeapPointableType list, const unsigned long index);

/**
 * Changes the @ref VuoLeapPointableType at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoLeapPointableType(const VuoList_VuoLeapPointableType list, const VuoLeapPointableType value, const unsigned long index);

/**
 * Inserts the `VuoLeapPointableType` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoLeapPointableType(const VuoList_VuoLeapPointableType list, const VuoLeapPointableType value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoLeapPointableType(VuoList_VuoLeapPointableType list, const VuoLeapPointableType value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoLeapPointableType(VuoList_VuoLeapPointableType list, const VuoLeapPointableType value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoLeapPointableType(VuoList_VuoLeapPointableType list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoLeapPointableType(VuoList_VuoLeapPointableType list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoLeapPointableType(VuoList_VuoLeapPointableType list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoLeapPointableType(VuoList_VuoLeapPointableType list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoLeapPointableType(VuoList_VuoLeapPointableType list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoLeapPointableType(VuoList_VuoLeapPointableType list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoLeapPointableType(VuoList_VuoLeapPointableType list);

/**
 * Removes the `VuoLeapPointableType` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoLeapPointableType(VuoList_VuoLeapPointableType list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoLeapPointableType(const VuoList_VuoLeapPointableType list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoLeapPointableType VuoList_VuoLeapPointableType_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoLeapPointableType_getJson(const VuoList_VuoLeapPointableType value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoLeapPointableType_getSummary(const VuoList_VuoLeapPointableType value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoLeapPointableType VuoList_VuoLeapPointableType_makeFromString(const char *str);
char * VuoList_VuoLeapPointableType_getString(const VuoList_VuoLeapPointableType value);
/// @}

/**
 * @}
 */

#endif
